==> linkv_rtc_im-0.3.6.tar.gz: ios/Classes/LVRTMEngine.m <==
//
//  LVRTMEngine.m
//  LinkV_Zego_iOS
//
//  Created by Wing on 2020/8/5.
//

#import "LVRTMEngine.h"

@interface LVRTMEngine ()<LVIMReceiveMessageDelegate>

@property (nonatomic, strong) LinkVRTCEngine *engine;
@property (nonatomic, strong) LVIMSDK *imSDK;

@property (nonatomic,weak) id<LVIMModuleEventDelegate, LVIMReceiveMessageDelegate, RoomCallbackProtocl> delegate;
@property (nonatomic,weak) id<RoomCallbackProtocl> roomMsgDelegate;

@property (nonatomic, copy) NSString *uid;

@end

@implementation LVRTMEngine

+ (instancetype)shared {
    static dispatch_once_t onceToken;
    static LVRTMEngine *_engine;
    dispatch_once(&onceToken, ^{
        _engine = [self new];
    });
    
    return _engine;
}

+ (instancetype)createEngine:(SDKType)type
                    rtcAppId:(NSString *)rtcAppId
                   rtcAppKey:(NSString *)rtcAppKey
                     imAppId:(NSString *)imAppId
                    imAppKey:(NSString *)imAppKey
                   isTestEnv:(BOOL)isTestEnv
          isInternationalEnv:(BOOL)isInternationalEnv
                    delegate:(id<RTCCallbackDelegate, LVIMModuleEventDelegate, LVIMReceiveMessageDelegate, RoomCallbackProtocl>)delegate {
    
    LVRTMEngine *rtm = [self shared];
    rtm.delegate = delegate;
    rtm.roomMsgDelegate = delegate;

    rtm.engine = [LinkVRTCEngine createEngine:type appId:rtcAppId appKey:rtcAppKey isTestEnv:isTestEnv isInternationalEnv:isInternationalEnv delegate:delegate];
    
    if (rtm.imSDK) {
        [rtm.imSDK setDebugMode:isTestEnv];
        [rtm setIMAuthEventDelegate:delegate];
        [rtm setIMGlobalReceiveMessageDelegate:delegate];
    } else {
        rtm.imSDK = [LVIMSDK sharedInstance];
        [rtm.imSDK initWithAppId:imAppId secret:imAppKey];
        [rtm.imSDK setDebugMode:isTestEnv];
        [rtm.imSDK start];
        
        [rtm setIMAuthEventDelegate:delegate];
        [rtm setIMGlobalReceiveMessageDelegate:delegate];
        [rtm setRoomMsgDelegate:delegate];
    }

    return rtm;
}

- (void)setLogOpen:(BOOL)isOpen {
    [self.imSDK setLogVisible:isOpen];
    [self.engine setLogOpen:isOpen];
}

#pragma mark - RTC
// 推流参数配置
- (void)setVideoConfig:(VideoConfigLevel)configLevel { [self.engine setVideoConfig:configLevel]; }
- (void)loginRoom:(NSString *)uid roomId:(NSString *)roomId isHost:(BOOL)isHost isAudioOnly:(BOOL)isAudioOnly {
    
    [self.imSDK setChatroomReceiveMessageDelegate:self];
    
    [self.imSDK joinChatroom:roomId context:nil callback:^(int ecode, int rcode, int64_t lvsgid, int64_t smsgid, int64_t stime, LVIMMessage *msg) {
        if (ecode == 0) {
            [self.engine loginRoom:uid roomId:roomId isHost:isHost isAudioOnly:isAudioOnly];
            
            [self.imSDK setChatroomReceiveMessageDelegate:self];
        } else {
            if ([self.delegate respondsToSelector:@selector(onRoomDisconnect:roomId:)]) {
                [self.delegate onRoomDisconnect:ecode roomId:roomId];
            }
        }
    }];
}
- (void)logoutRoom {
    NSString *roomId = self.engine.roomId;
    [self.engine logoutRoom];
    [self.imSDK leaveChatroom:roomId context:nil callback:nil];
    [self.imSDK setChatroomReceiveMessageDelegate:nil];
}

- (void)startPreview:(UIView *)view {
    [self.engine startPreview:view];
}
- (void)startPublishing {
    [self.engine startPublishing];
}
- (void)startPlayingStream:(NSString *)uid inView:(UIView *)view {
    [self.engine startPlayingStream:uid inView:view];
}
- (void)stopPlayingStream:(NSString *)uid {
    [self.engine stopPlayingStream:uid];
}
// 转换前后置摄像头
- (void)useFrontCamera:(BOOL)isFront {
    [self.engine useFrontCamera:isFront];
}
// 获取连麦SDK的名称
- (NSString *)getSolutionName {
    return [self.engine getSolutionName];
}
// 获取支持连麦数
- (int)getSupportLine {
    return [self.engine getSupportLine];
}
// 获取SDK类型 3为自研，4为zego
- (SDKType)getSDKType {
    return [self.engine getSDKType];
}
// 设置是否静音推流
- (void)enableMic:(BOOL)isMute {
    [self.engine enableMic:isMute];
}
// 释放引擎
- (void)unInitSDK {
    [self.engine unInitSDK];
}
- (int)getMicVolume {
    return [self.engine getMicVolume];
}

- (NSString *)sdkVersion {
    return [self.engine sdkVersion];
}

/// 设置推流时数据统计信息回调的频率，回调方法请看 OnPublishQualityUpdate 回调
/// @param seconds 回调频率（1 / sec）
- (void)setPublishQualityMonitorCycle:(int)seconds {
    [self.engine setPublishQualityMonitorCycle:seconds];
}
/// 设置拉流时数据统计信息回调的频率，回调方法请看 OnPlayQualityUpate 回调
/// @param seconds 回调频率（1 / sec）
- (void)setPlayQualityMonitorCycle:(int)seconds {
    [self.engine setPlayQualityMonitorCycle:seconds];
}

/// 停止发布音视频数据
- (void)stopPublishStream {
    [self.engine stopPublishStream];
}

/// 设置是否允许 SDK 自动根据设备的方向调整视频的输出方向
/// @param enable 打开和关闭（默认是 YES）
-(void)enableVideoAutoRotation:(BOOL)enable {
    [self.engine enableVideoAutoRotation:enable];
}

/**
 * 设置美颜滤镜级别
 * @param beautyLevel 滤镜美颜级别 取值范围[0,1]，设置-1关闭
 */
- (void)setBeautyLevel:(CGFloat)beautyLevel {
    [self.engine setBeautyLevel:beautyLevel];
}
/**
 * 设置滤镜明亮度
 * @param brightLevel 滤镜明亮度 取值范围[0,1]，设置-1关闭
 */
- (void)setBrightLevel:(CGFloat)brightLevel {
    [self.engine setBrightLevel:brightLevel];
}
/**
 * 设置滤镜饱和度
 * @param toneLevel 滤镜饱和度 取值范围[0,1]，设置-1关闭
 */
- (void)setToneLevel:(CGFloat)toneLevel {
    [self.engine setToneLevel:toneLevel];
}

/// 开始采集视频流
- (void)startCapture {
    [self.engine startCapture];
}
/// 停止采集视频流
- (void)stopCapture {
    [self.engine stopCapture];
}

#pragma mark - IM
- (int)loginIM:(NSString *)uid {
    self.uid = uid;
    return [self.imSDK login:uid];
}

// 设置IM鉴权监听
- (void)setIMAuthEventDelegate:(id<LVIMModuleEventDelegate>)delegate {
    [self.imSDK setModuleEventDelegate:delegate];
}

// 设置全局消息监听
- (void)setIMGlobalReceiveMessageDelegate:(id<LVIMReceiveMessageDelegate>)delegate {
    [self.imSDK setGlobalReceiveMessageDelegate:delegate];
}

- (void)setRoomMsgDelegate:(id<RoomCallbackProtocl>)delegate {
    _roomMsgDelegate = delegate;
    
    [self.engine performSelector:@selector(setDelegate:) withObject:delegate];
}

- (void)sendPrivateMessage:(LVIM_MESSAGE_SUBTYPE)subType
                       tid:(NSString *)tid
                      type:(NSString *)type
                   content:(NSString *)content
                  complete:(LVIM_SEND_MESSAGE_CALLBACK)complete {
    [self sendPrivateMessage:subType tid:tid type:type content:content pushTitle:nil pushContent:nil pushExtra:nil extend3:nil extend4:nil targetAppID:nil targetAppUID:nil complete:complete];
}

- (void)sendPrivateMessage:(LVIM_MESSAGE_SUBTYPE)subType
                      tid:(NSString *)tid
                     type:(NSString *)type
                  content:(NSString *)content
                pushTitle:(NSString *)pushTitle
              pushContent:(NSString *)pushContent
                 pushExtra:(NSString *)pushExtra
                  extend3:(NSString *)extend3
                  extend4:(NSData *)extend4
              targetAppID:(NSString *)targetAppID
             targetAppUID:(NSString *)targetAppUID
                 complete:(LVIM_SEND_MESSAGE_CALLBACK)complete {
    
    LVIMMessage *msg = [LVIMMessage new];
    LVPushContent *push;
    if (pushTitle || pushContent || pushExtra) {
        push = [LVPushContent new];
        push.title = pushTitle;
        push.body = pushContent;
        push.extra = pushExtra;
    }
    
    int result = [msg buildPrivateRequest:subType fid:self.uid tid:tid type:type ?: @"" content:content pushContent:push extend3:extend3 extend4:extend4 targetAppID:targetAppID targetAppUID:targetAppUID];
    
    if (result != 0) {
        if (complete) {
            complete(ERROR_BUILD_MSG, 0, 0, 0, 0, nil);
        }
        return;
    }
    
    [self.imSDK sendMessage:msg context:nil callback:complete];
}

/**
 * 发送房间消息
 * @param roomId   房间Id
 * @param content 消息内容
 * @param complete   发送结果监听
 */
- (void)sendRoomMessage:(NSString *)roomId content:(NSString *)content complete:(LVIM_SEND_MESSAGE_CALLBACK)complete {
    [self sendRoomMessage:roomId content:content msgType:@"msgTye" complete:complete];
}

/**
 * 发送房间消息
 * @param roomId   房间Id
 * @param content 消息内容
 * @param msgType 消息的自定义扩展类型，可作为消息标识，但不能传空或者空串
 * @param complete   发送结果监听
 */
- (void)sendRoomMessage:(NSString *)roomId content:(NSString *)content msgType:(NSString *)msgType complete:(LVIM_SEND_MESSAGE_CALLBACK)complete {
    
    LVIMMessage *msg = [LVIMMessage new];
    int result = [msg buildChatroomRequest:self.uid tid:roomId type:msgType content:content];
    
    if (result != 0) {
        if (complete) {
            complete(ERROR_BUILD_MSG, 0, 0, 0, 0, nil);
        }
        return;
    }
    
    result = [self.imSDK sendMessage:msg context:nil callback:complete];
    if (result != 0 && complete) {
        complete(result, 0, 0, 0, 0, msg);
    }
}

#pragma mark - LVIMReceiveMessageDelegate 房间消息
- (BOOL)onIMReceiveMessageFilter:(int32_t)diytype fromid:(const char *)fromid toid:(const char *)toid msgtype:(const char *)msgtype content:(const char *)content waitings:(int)waitings packetSize:(int)packetSize waitLength:(int)waitLength bufferSize:(int)bufferSize {
    NSString *to = [NSString stringWithUTF8String:toid];
    return ![to isEqualToString:self.engine.roomId];
}

- (int)onIMReceiveMessageHandler:(NSString *)owner immsg:(LVIMMessage *)immsg waitings:(int)waitings packetSize:(int)packetSize waitLength:(int)waitLength bufferSize:(int)bufferSize {
    
    if ([self.roomMsgDelegate respondsToSelector:@selector(onRoomMessageReceive:)]) {
        return [self.roomMsgDelegate onRoomMessageReceive:immsg];
    }
    
    return 0;
}

@end

==> linkv_rtc_im-0.3.10.tar.gz: ios/Classes/LVRTMEngine.m <==
//
//  LVRTMEngine.m
//  LinkV_Zego_iOS
//
//  Created by Wing on 2020/8/5.
//

#import "LVRTMEngine.h"

@interface LVRTMEngine ()<LVIMReceiveMessageDelegate>

@property (nonatomic, strong) LinkVRTCEngine *engine;
@property (nonatomic, strong) LVIMSDK *imSDK;

@property (nonatomic,weak) id<LVIMModuleEventDelegate, LVIMReceiveMessageDelegate, RoomCallbackProtocl> delegate;
@property (nonatomic,weak) id<RoomCallbackProtocl> roomMsgDelegate;

@property (nonatomic, copy) NSString *uid;

@end

@implementation LVRTMEngine

+ (instancetype)shared {
    static dispatch_once_t onceToken;
    static LVRTMEngine *_engine;
    dispatch_once(&onceToken, ^{
        _engine = [self new];
    });
    
    return _engine;
}

//rtc + im
+ (instancetype)createEngine:(SDKType)type
                    rtcAppId:(NSString *)rtcAppId
                   rtcAppKey:(NSString *)rtcAppKey
                     imAppId:(NSString *)imAppId
                    imAppKey:(NSString *)imAppKey
                   isTestEnv:(BOOL)isTestEnv
          isInternationalEnv:(BOOL)isInternationalEnv
                    delegate:(id<RTCCallbackDelegate, LVIMModuleEventDelegate, LVIMReceiveMessageDelegate, RoomCallbackProtocl>)delegate
             imStatusHandler:(void(^)(BOOL))imStatusHandler{
    
    LVRTMEngine *rtm = [self shared];
    rtm.delegate = delegate;
    rtm.roomMsgDelegate = delegate;

    rtm.engine = [LinkVRTCEngine createEngine:type appId:rtcAppId appKey:rtcAppKey isTestEnv:isTestEnv isInternationalEnv:isInternationalEnv delegate:delegate];
    
    bool imStatus = false;
    
    if (rtm.imSDK) {
        [rtm.imSDK setDebugMode:isTestEnv];
        [rtm setIMAuthEventDelegate:delegate];
        [rtm setIMGlobalReceiveMessageDelegate:delegate];
    } else {
        rtm.imSDK = [LVIMSDK sharedInstance];
        imStatus = [rtm.imSDK initWithAppId:imAppId secret:imAppKey];
        [rtm.imSDK setDebugMode:isTestEnv];
        [rtm.imSDK start];
        
        [rtm setIMAuthEventDelegate:delegate];
        [rtm setIMGlobalReceiveMessageDelegate:delegate];
        [rtm setRoomMsgDelegate:delegate];
    }
    imStatusHandler(imStatus);

    return rtm;
}

//rtc init
+ (instancetype)createEngine:(SDKType)type
                    rtcAppId:(NSString *)rtcAppId
                   rtcAppKey:(NSString *)rtcAppKey
                   isTestEnv:(BOOL)isTestEnv
          isInternationalEnv:(BOOL)isInternationalEnv
                    delegate:(id<RTCCallbackDelegate, LVIMModuleEventDelegate, LVIMReceiveMessageDelegate, RoomCallbackProtocl>)delegate {
    
    LVRTMEngine *rtm = [self shared];
    rtm.delegate = delegate;
    rtm.roomMsgDelegate = delegate;

    rtm.engine = [LinkVRTCEngine createEngine:type appId:rtcAppId appKey:rtcAppKey isTestEnv:isTestEnv isInternationalEnv:isInternationalEnv delegate:delegate];
    return rtm;
}

//im
+ (instancetype)createEngine:(SDKType)type
                     imAppId:(NSString *)imAppId
                    imAppKey:(NSString *)imAppKey
                   isTestEnv:(BOOL)isTestEnv
                    delegate:(id<RTCCallbackDelegate, LVIMModuleEventDelegate, LVIMReceiveMessageDelegate, RoomCallbackProtocl>)delegate
             imStatusHandler:(void(^)(BOOL))imStatusHandler{
    
    LVRTMEngine *rtm = [self shared];
    rtm.delegate = delegate;
    rtm.roomMsgDelegate = delegate;

    bool imStatus = false;
    if (rtm.imSDK) {
        [rtm.imSDK setDebugMode:isTestEnv];
        [rtm setIMAuthEventDelegate:delegate];
        [rtm setIMGlobalReceiveMessageDelegate:delegate];
    } else {
        rtm.imSDK = [LVIMSDK sharedInstance];
        imStatus = [rtm.imSDK initWithAppId:imAppId secret:imAppKey];
        [rtm.imSDK setDebugMode:isTestEnv];
        [rtm.imSDK start];
        
        [rtm setIMAuthEventDelegate:delegate];
        [rtm setIMGlobalReceiveMessageDelegate:delegate];
        [rtm setRoomMsgDelegate:delegate];
    }
    
    imStatusHandler(imStatus);

    return rtm;
}

- (void)setLogOpen:(BOOL)isOpen {
    [self.imSDK setLogVisible:isOpen];
    [self.engine setLogOpen:isOpen];
}

#pragma mark - RTC
// 推流参数配置
- (void)setVideoConfig:(VideoConfigLevel)configLevel { [self.engine setVideoConfig:configLevel]; }
- (void)loginRoom:(NSString *)uid roomId:(NSString *)roomId isHost:(BOOL)isHost isAudioOnly:(BOOL)isAudioOnly {
    
    [self.imSDK setChatroomReceiveMessageDelegate:self];
    
    [self.imSDK.chatroomManager joinChatroom:roomId context:nil callback:^(int ecode, int rcode, int64_t lvsgid, int64_t smsgid, int64_t stime, LVIMMessage *msg) {
        if (ecode == 0) {
            [self.engine loginRoom:uid roomId:roomId isHost:isHost isAudioOnly:isAudioOnly];
            
            [self.imSDK setChatroomReceiveMessageDelegate:self];
        } else {
            if ([self.delegate respondsToSelector:@selector(onRoomDisconnect:roomId:)]) {
                [self.delegate onRoomDisconnect:ecode roomId:roomId];
            }
        }
    }];
}
- (void)logoutRoom {
    NSString *roomId = self.engine.roomId;
    [self.engine logoutRoom];
    [self.imSDK.chatroomManager leaveChatroom:roomId context:nil callback:nil];
    [self.imSDK setChatroomReceiveMessageDelegate:nil];
}

- (void)startPreview:(UIView *)view {
    [self.engine startPreview:view];
}
- (void)startPublishing {
    [self.engine startPublishing];
}
- (void)startPlayingStream:(NSString *)uid inView:(UIView *)view {
    [self.engine startPlayingStream:uid inView:view];
}
- (void)stopPlayingStream:(NSString *)uid {
    [self.engine stopPlayingStream:uid];
}
// 转换前后置摄像头
- (void)useFrontCamera:(BOOL)isFront {
    [self.engine useFrontCamera:isFront];
}
// 获取连麦SDK的名称
- (NSString *)getSolutionName {
    return [self.engine getSolutionName];
}
// 获取支持连麦数
- (int)getSupportLine {
    return [self.engine getSupportLine];
}
// 获取SDK类型 3为自研，4为zego
- (SDKType)getSDKType {
    return [self.engine getSDKType];
}
// 设置是否静音推流
- (void)enableMic:(BOOL)isMute {
    [self.engine enableMic:isMute];
}
// 释放引擎
- (void)unInitRtcAndImSDK {
    if (self.engine != nil) {
        [self.engine unInitSDK];
    }
}

// 释放引擎
- (void)unInitRtcSDK {
    if (self.engine != nil) {
        [self.engine unInitSDK];
    }
}

// 释放引擎
- (void)unInitImSDK {
  
}

- (int)getMicVolume {
    return [self.engine getMicVolume];
}

- (NSString *)sdkVersion {
    return [self.engine sdkVersion];
}

/// 设置推流时数据统计信息回调的频率，回调方法请看 OnPublishQualityUpdate 回调
/// @param seconds 回调频率（1 / sec）
- (void)setPublishQualityMonitorCycle:(int)seconds {
    [self.engine setPublishQualityMonitorCycle:seconds];
}
/// 设置拉流时数据统计信息回调的频率，回调方法请看 OnPlayQualityUpate 回调
/// @param seconds 回调频率（1 / sec）
- (void)setPlayQualityMonitorCycle:(int)seconds {
    [self.engine setPlayQualityMonitorCycle:seconds];
}

/// 停止发布音视频数据
- (void)stopPublishStream {
    [self.engine stopPublishStream];
}

/// 设置是否允许 SDK 自动根据设备的方向调整视频的输出方向
/// @param enable 打开和关闭（默认是 YES）
-(void)enableVideoAutoRotation:(BOOL)enable {
    [self.engine enableVideoAutoRotation:enable];
}

/**
 * 设置美颜滤镜级别
 * @param beautyLevel 滤镜美颜级别 取值范围[0,1]，设置-1关闭
 */
- (void)setBeautyLevel:(CGFloat)beautyLevel {
    [self.engine setBeautyLevel:beautyLevel];
}
/**
 * 设置滤镜明亮度
 * @param brightLevel 滤镜明亮度 取值范围[0,1]，设置-1关闭
 */
- (void)setBrightLevel:(CGFloat)brightLevel {
    [self.engine setBrightLevel:brightLevel];
}
/**
 * 设置滤镜饱和度
 * @param toneLevel 滤镜饱和度 取值范围[0,1]，设置-1关闭
 */
- (void)setToneLevel:(CGFloat)toneLevel {
    [self.engine setToneLevel:toneLevel];
}

/// 开始采集视频流
- (void)startCapture {
    [self.engine startCapture];
}
/// 停止采集视频流
- (void)stopCapture {
    [self.engine stopCapture];
}

#pragma mark - IM
- (int)loginIM:(NSString *)uid loginType:(LVIM_LOGIN_TYPE)loginType {
    self.uid = uid;
    return [self.imSDK login:uid loginType:loginType];
}

// 设置IM鉴权监听
- (void)setIMAuthEventDelegate:(id<LVIMModuleEventDelegate>)delegate {
    [self.imSDK setModuleEventDelegate:delegate];
}

// 设置全局消息监听
- (void)setIMGlobalReceiveMessageDelegate:(id<LVIMReceiveMessageDelegate>)delegate {
    [self.imSDK setGlobalReceiveMessageDelegate:delegate];
}

- (void)setRoomMsgDelegate:(id<RoomCallbackProtocl>)delegate {
    _roomMsgDelegate = delegate;
    
    [self.engine performSelector:@selector(setDelegate:) withObject:delegate];
}

- (void)sendPrivateMessage:(LVIM_MESSAGE_SUBTYPE)subType
                       tid:(NSString *)tid
                      type:(NSString *)type
                   content:(NSString *)content
                  complete:(LVIM_SEND_MESSAGE_CALLBACK)complete {
    [self sendPrivateMessage:subType tid:tid type:type content:content pushTitle:nil pushContent:nil pushExtra:nil extend3:nil extend4:nil targetAppID:nil targetAppUID:nil complete:complete];
}

- (void)sendPrivateMessage:(LVIM_MESSAGE_SUBTYPE)subType
                      tid:(NSString *)tid
                     type:(NSString *)type
                  content:(NSString *)content
                pushTitle:(NSString *)pushTitle
              pushContent:(NSString *)pushContent
                 pushExtra:(NSString *)pushExtra
                  extend3:(NSString *)extend3
                  extend4:(NSData *)extend4
              targetAppID:(NSString *)targetAppID
             targetAppUID:(NSString *)targetAppUID
                 complete:(LVIM_SEND_MESSAGE_CALLBACK)complete {
    
    LVIMMessage *msg = [LVIMMessage new];
    LVPushContent *push;
    if (pushTitle || pushContent || pushExtra) {
        push = [LVPushContent new];
        push.title = pushTitle;
        push.body = pushContent;
        push.extra = pushExtra;
    }
    
    int result = [msg buildPrivateRequest:subType fid:self.uid tid:tid type:type ?: @"" content:content pushContent:push extend3:extend3 extend4:extend4 targetAppID:targetAppID targetAppUID:targetAppUID];
    
    if (result != 0) {
        if (complete) {
            complete(ERROR_BUILD_MSG, 0, 0, 0, 0, nil);
        }
        return;
    }
    
    [self.imSDK sendMessage:msg context:nil callback:complete];
}

/**
 * 发送房间消息
 * @param roomId   房间Id
 * @param content 消息内容
 * @param complete   发送结果监听
 */
- (void)sendRoomMessage:(NSString *)roomId content:(NSString *)content complete:(LVIM_SEND_MESSAGE_CALLBACK)complete {
    [self sendRoomMessage:roomId content:content msgType:@"msgTye" complete:complete];
}

/**
 * 发送房间消息
 * @param roomId   房间Id
 * @param content 消息内容
 * @param msgType 消息的自定义扩展类型，可作为消息标识，但不能传空或者空串
 * @param complete   发送结果监听
 */
- (void)sendRoomMessage:(NSString *)roomId content:(NSString *)content msgType:(NSString *)msgType complete:(LVIM_SEND_MESSAGE_CALLBACK)complete {
    
    LVIMMessage *msg = [LVIMMessage new];
    int result = [msg buildChatroomRequest:self.uid tid:roomId type:msgType content:content];
    
    if (result != 0) {
        if (complete) {
            complete(ERROR_BUILD_MSG, 0, 0, 0, 0, nil);
        }
        return;
    }
    
    result = [self.imSDK sendMessage:msg context:nil callback:complete];
    if (result != 0 && complete) {
        complete(result, 0, 0, 0, 0, msg);
    }
}

#pragma mark - LVIMReceiveMessageDelegate 房间消息
- (BOOL)onIMReceiveMessageFilter:(int32_t)diytype fromid:(const char *)fromid toid:(const char *)toid msgtype:(const char *)msgtype content:(const char *)content waitings:(int)waitings packetSize:(int)packetSize waitLength:(int)waitLength bufferSize:(int)bufferSize {
    NSString *to = [NSString stringWithUTF8String:toid];
    return ![to isEqualToString:self.engine.roomId];
}

- (int)onIMReceiveMessageHandler:(NSString *)owner immsg:(LVIMMessage *)immsg waitings:(int)waitings packetSize:(int)packetSize waitLength:(int)waitLength bufferSize:(int)bufferSize {
    
    if ([self.roomMsgDelegate respondsToSelector:@selector(onRoomMessageReceive:)]) {
        return [self.roomMsgDelegate onRoomMessageReceive:immsg];
    }
    
    return 0;
}

@end
